import os
# By AWARE on RC: python3.6 and it happy this *local* version of dataclasses, but ver>3.6 should go with system-wide version
try:
    from dataclasses import dataclass, field
except:
    from dataclasses_local import dataclass, field
from pathlib      import Path

from .services.remote_interfaces import RemoterInterface
from .services.sshhandler   import SSHHandler
from .services.sambahandler import SambaHandler
from .services.firebase     import Firebase
from .services.scp          import Scp_wrap

from .tools.decor  import with_no_internal_error, InternalError, Colors

from .services.status import Status

class RemoterError(Exception): pass

'''
The class for communication with remote devices
'''
@dataclass
class Remoter:
    __version__  = "1.0.0"
    debug                 : bool             = False
    verbose               : bool             = False
    internal_name         : str              = "[Remoter]"
    internal_error        : InternalError    = InternalError()
    # tmp folder
    _internal_path        : Path             = Path("./.internal/")
    _connection           : RemoterInterface = None

    connection_type       : str              = None # "ssh/samba/cloud"
    local_storage         : Path             = None
    remote_home           : Path             = None
    #
    ssh_key               : Path             = None
    username              : str              = "username"
    host                  : str              = "192.168.2.52"
    psw                   : str              = ""
    port                  : int              = 22
    #
    additional_files      : list             = field(default_factory=list)

    # Samba and Firebase 
    build_type            : str              = None # debug/release  #used as a domain
    
    # Samba specific
    mount_remote_str       : str             = None
    mount_local_path       : str             = None

    # Firebase specific
    release_shortcut      : str              = None # EE/...
    state                 : str              = None # Alpha/Beta/Stable

    def __post_init__(self):

        assert isinstance(self.host, str)
        #
        if self.local_storage is not None:
            if not self.local_storage.exists():
                self.local_storage.mkdir(parents=True, exist_ok=True)

        # establish connection with remote
        if self.connection_type:
            if self.connection_type == "ssh":
                self._connection = SSHHandler(
                    verbose  = self.verbose, 
                    debug    = self.debug,
                    host     = self.host,
                    username = self.username,
                    psw      = self.psw,
                    port     = self.port,
                    ssh_key  = self.ssh_key,
                )
            elif self.connection_type == "samba":
                self._connection = SambaHandler(
                    debug            = self.debug, 
                    host             = self.host,
                    username         = self.username,
                    psw              = self.psw,
                    port             = self.port,
                    mount_remote_str = self.mount_remote_str,
                    mount_local_path = self.mount_local_path,
                    build_type       = self.build_type,      
                )
            elif self.connection_type == "firebase":
                self._connection = Firebase(
                    debug            = self.debug,
                    build_type       = self.build_type,       # debug/release
                    release_shortcut = self.release_shortcut, # EE/...
                    state            = self.state,            # Alpha/Beta/Stable
                )
            elif self.connection_type == "ssh_wrap":
                self._connection = Scp_wrap(
                    verbose = self.verbose,
                    debug   = self.debug,
                    host    = self.host,
                    username= self.username,
                    port    = self.port,
                    psw     = self.psw
                )
            else:
                raise RemoterError(f"{self.connection_type} is not implemented")
        else:
            raise RemoterError("`connection_type` should be provided")
        self.notify()

    def __str__(self) -> str:
        return '\n'.join("%s: %s" % item for item in vars(self).items())

    def connect(self):
        print(f"[INFO] {self._connection.__class__.__name__} connecting...")
        return self._connection.connect()
    
    def disconnect(self):
        print(f"[INFO] {self._connection.__class__.__name__} disconnecting...")
        return self._connection.disconnect()

    def share_connection(self) -> RemoterInterface: return self._connection

    def notify(self) -> None:
        print(f"[INFO] => {__class__.__name__} [{self.host}] is initialized.")
        if self.debug: print(f"=> Handler: {self._connection}")

    '''
    Remove something redundant...
    '''
    def sanitize(self) -> None:
        raise RemoterError(f"{Colors.RED}{self.internal_name} WARNING!!! `Sanitize` is not implemented for [{self.method_specification}] method yet :({Colors.END}")

    '''
    Upload to remote a file with a path
    '''
    @with_no_internal_error
    def upload(self, src_fname:Path = None, dst_fname:Path = None) -> Status:
        return self._connection.upload(src_fname=src_fname, dst_fname=dst_fname)

    '''
    Download a file to local
    '''
    @with_no_internal_error
    def download(self, src_fname:Path = None, dst_fname:Path = None) -> Status:
        return self._connection.download(src_fname=src_fname, dst_fname=dst_fname)
    
